/*--------------------------------------------------------------------------------------------------
//
//     File Name:   StuffItSDK.h
//
//   Description:   Interfaces for accessing the StuffIt Engine.
//
//--------------------------------------------------------------------------------------------------
//
//  Copyright (c) 1992-2004 Allume Systems, Inc.
//
//  This source code and specific concepts contained herein are Confidential
//  Information and Property of Allume Systems.
//  Distribution is prohibited without written permission of Allume Systems.
//
//--------------------------------------------------------------------------------------------------
//
//  Allume Systems, Inc.                   http://www.allume.com/
//  245 Westridge Drive                    (831) 761-6200
//  Watsonville, CA  95076                 (831) 761-6206 FAX
//
//--------------------------------------------------------------------------------------------------
//  $Id: StuffItSDK.h 306 2006-10-06 03:14:54Z glenn $
//------------------------------------------------------------------------------------------------*/

/*!
	\mainpage StuffIt Engine SDK
	
	The StuffIt Engine SDK is a cross-platform suite of APIs allowing a client to access
	the StuffIt Engine for basic \ref compress and \ref expand as well as more advanced
	tasks such as \ref manipulation of an archive's contents.
	
	In order for the SDK to be cross platform, certain data types and object representations
	needed to be abstracted. \ref strings and references to \ref fsobjects need to be converted
	from their platform specific types to their respective StuffIt SDK types.
	
	\section strings Strings
	
	The StringRef is used to provide strings to the SDK where necessary. A StringRef can
	be created a number in a number of ways that is dependant on the platform the SDK is 
	executing on. For instance, a c string or pascal string may be used to create a StringRef on
	Windows or MacOS, but only the MacOS supports creation of a StringRef from a CFString.

	See StuffItSDKStrings.h.
	
	\section fsobjects File System Objects
	
	An #FSObjectRef is the sole way of providing objects to the SDK that represent an item
	within the file system. Similar to #StringRef, creation of an #FSObjectRef varies
	by platform.
	
	See StuffItSDKFiles.h.

	\section notifications Notifications
	
	The SDK uses notifications so that the client can provide it's own progress user interface
	as well as other interactions that might require user feedback.
	
	See StuffItSDKNotifications.h and StuffItSDKOptions.h.

	\section ui User Interface
	
	\deprecated
	The SDK can display user interface in four different areas, progress, password entry,
	destination selection, and return receipt handling. User Interface can be entirely
	suppressed by setting the EngineStatusCallbackProcPtr to SuppressStatusProc. See 
	StuffItSDKCallbacks.h and StuffItSDKOptions.h.
	
	As of the 0x820 version, the SDK will no longer provide any user interface. On Mac OS X, the SDK links against
	CoreServices.framework allowing it to be used from many different contexts such as command line tools and
	GUI apps.
	
	The SDK provides a notification mechanism defined in StuffItSDKNotifications.h for getting status during an operation
	or when it's necessary for information to be supplied during an expansion or compression.
	
	\section threads Thread Issues
	
	As of version 0x810, the SDK can be used from within a preemptive thread. When called from such
	a thread, any user interface that the engine might display will be suppressed. Typically, a client
	would use notifications to handle any user feedback.

	\section examples Examples
	Examples of SDK Usage can be found \ref page1 here.
*/


/*!
	\file StuffItSDK.h
	
	\brief Interfaces for accessing opening and closing the StuffIt Engine and for determining
	version information and registration.
*/

#ifndef STUFFITSDK_H
#define STUFFITSDK_H

#ifndef STUFFITSDKCONFIG_H
#include <StuffIt/StuffItSDKConfig.h>
#endif

#ifndef STUFFITSDKTYPES_H
#include <StuffIt/StuffItSDKTypes.h>
#endif

#ifndef STUFFITSDKERRORS_H
#include <StuffIt/StuffItSDKErrors.h>
#endif

#if TARGET_RT_MAC_CFM
	#ifndef __CODEFRAGMENTS__
	#include <CodeFragments.h>
	#endif
#endif

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT
#pragma import on
#endif

/*! \brief Useful for supplying no registration.

*/
#define	kNoRegistrationNumber		nil

/*! \brief Useful version constants.
    
    This is the version of the library you know about the top byte is the version, the bottom is the
    revision. See documentation for how to use this information.
*/

enum LibraryVersion {
    kCurrentLibraryVersion                  =   0x0820,		/**< kCurrentLibraryVersion Current version of the library. */
    kFirstSupportedEngine                   =   0x0700		/**< kFirstSupportedEngine Earliest version of the StuffIt Engine that can be used with the SDK. */ 
};

/*! \brief A reference to the StuffItEngine.

	An EngineRef is returned by OpenStuffItEngine and is used
	by many of the APIs throughout the SDK. It is disposed of
	with CloseStuffItEngine.

*/
typedef struct OpaqueEngineObject *         EngineRef;

/**/
/*   Engine version checking and availability                                          */
/**/

/*!
	\brief Get the version of the StuffIt Engine SDK
	
	Determine the version of the StuffIt Engine SDK.
	
	\return Version of the StuffIt Engine SDK
*/

extern _PASCAL_  int16_t
GetStuffItEngineLibraryVersion( void );

/*! \brief Get the verion of the StuffIt Engine SDK
	
	Determine the major version number, minor version number, patch level and build number 
    of the StuffIt Engine SDK

	\param inEngineRef Reference obtained from OpenStuffItEngine.
	\param outMajor Pointer to uint16_t representing a major version number. Null is invalid.
	\param outMinor Pointer to uint16_t representing a minor version number. Null is invalid.
	\param outPatchLevel Pointer to uint16_t representing a patch level. Null is invalid.
	\param outBuild Pointer to uint16_t representing a build number. Null is invalid.
	\return Returns an EngineError
*/
extern _PASCAL_ EngineError 
GetVersionInfo(EngineRef inEngineRef, uint16_t* outMajor, uint16_t* outMinor, uint16_t* outPatchLevel, uint16_t* outBuild);

/*! \brief Determine if the StuffIt Engine is registered
	
	Determine if the StuffIt Engine is registered

	\return True if the StuffIt Engine is registered.
*/
extern _PASCAL_ Boolean
IsStuffItEngineRegistered( void ) WEAK_LINK;

/**/
/*   Engine Open and Close                                                             */
/**/
/*! \brief Opens the StuffIt Engine
	
	Initializes the StuffIt Engine and returns a reference used throughout
	the sdk.

	\param outEngineRef Reference returned if the engine has opened successfully. Null is invalid.
	\param registrationCode License code provided by Allume
	\return Returns an EngineError
*/
extern _PASCAL_  EngineError
OpenStuffItEngine(
    EngineRef *     outEngineRef,
    const char *	registrationCode ) WEAK_LINK;

/*! \brief Closes the StuffIt Engine
	
	Closes the StuffIt Engine and releases any resources created by it.
	
	\param inEngineRef Reference obtained from OpenStuffItEngine. Null is invalid.
	\return Returns an EngineError
*/
extern _PASCAL_  EngineError
CloseStuffItEngine(
    EngineRef       inEngineRef ) WEAK_LINK;


#if TARGET_RT_MAC_CFM
    #ifdef __cplusplus
        inline Boolean StuffItEngineLibraryAvailable() { return ((GetStuffItEngineLibraryVersion != (void*)kUnresolvedCFragSymbolAddress)); }
    #else
        #define StuffItEngineLibraryAvailable()  ((GetStuffItEngineLibraryVersion != (void*)kUnresolvedCFragSymbolAddress))
    #endif
#elif TARGET_RT_MAC_MACHO
	extern _PASCAL_ Boolean StuffItEngineLibraryAvailable() WEAK_LINK;
#elif TARGET_OS_WIN32
    #ifdef __cplusplus
        inline Boolean StuffItEngineLibraryAvailable() { return true; }	// TODO: Needs to change for MachO
    #else
        #define StuffItEngineLibraryAvailable()  (true)	// TODO: Needs to change for MachO
    #endif
#else
	#error Unknown Compiler Target
#endif

#ifdef PRAGMA_IMPORT_OFF
#pragma import off
#elif PRAGMA_IMPORT
#pragma import reset
#endif

#ifdef __cplusplus
}
#endif

#endif

/*!
	\page page1 Examples
	\section compress Compression
	
	The following examples illustrate compression and expansion. They use Mac platform specific types
	for file system representation, but that is easily replaced with other representations.
	
	
	\code
	void ArchiveFiles(const std::vector<FSSpec> &fileList)
	{
		EngineRef 	engineRef;
		EngineError	err = kStuffItErrorNoError;
		
		// Open the StuffIt Engine. Passing nil for the reg code param will return an error
		// if the user does not have registered version of deluxe installed.
		err = OpenStuffItEngine(&engineRef, nil);
		
		FSObjectListRef		filesToCompress;
		
		// Create a list of file system objects for the engine to compress.
		err = NewFSObjectList(mEngineRef, &filesToCompress);
		
		for (std::vector<FSSpec>::iterator index = fileList.begin(); index != fileList.end(); ++index)
			{
			err = AddFSSpecToFSObjectList(engineRef, *index, filesToCompress);
			}
		
		// Create a StuffItX archive in the same location as the source files
		// 
		// Uncomment to encode the archive with binhex. The first line is the preferred mechanism
		// for accessing options but is only available in sdk 8 and later. The second line is
		// available from 7.0 on and will eventually be removed.

		//err = SetEngineFormatIDProperty(engineRef, kEnginePropertyIDPostEncodingFormat, kFormatIDBinHex);
		//err = SetPostEncodingFormat(engineRef, kFormatIDBinHex);

		err = ArchiveFSObjectList(engineRef, kFormatIDStuffItX, filesToCompress, nil, nil, nil);
		
		// Everything that starts with New... has a corresponding Dispose. Similarly, everything that
		// starts with Acquire... has a paired Release.
		err = DisposeFSObjectList(filesToCompress);
		
		err = CloseStuffItEngine(engineRef);
	}
	\endcode
	
	\section expand Expansion

	\code
	void ExpandFile(const FSSpec &archive)
	{
		EngineRef 	engineRef;
		EngineError	err = kStuffItErrorNoError;
		
		// Open the StuffIt Engine. Passing nil for the reg code param will return an error
		// if the user does not have registered version of deluxe installed.
		err = OpenStuffItEngine(&engineRef, nil);
		
		FSObjectRef		archiveToExpand;
		
		// Create file system object for the engine to expand.
		err = NewFSObjectFromFSSpec(archive, &archiveToExpand);

		// Expand an archive in the same location as the source
		err = ExpandOrDecodeFile(engineRef, archiveToExpand, nil, nil, nil);
		
		// Everything that starts with New... has a corresponding Dispose. Similarly, everything that
		// starts with Acquire... has a paired Release.
		err = DisposeFSObject(filesToCompress);
		
		err = CloseStuffItEngine(engineRef);
	}

	\endcode
	
	\section manipulation Archive browsing

	Archive browsing and manipulation is very similar to accessing a hierarchical file system. An archive
	always has a root folder object which can be iterated, and its contents examined.
	
	\code
	void IterateArchiveRoot(EngineRef engine, FSObjectRef archiveFile)
	{
		EngineError	err = kStuffItErrorNoError;
		
		//Before opening an archive it must be acquired and eventually released.
		ArchiveRef	archiveRef;
		err = AcquireArchive(engine, archiveFile, &archiveRef);
		
		//Open the archive. Archives can be opened in read-only or read-write.
		//If the archive is encrypted, use #OpenEncryptedArchive
		err = OpenArchive(engine, archiveRef, kFullMode);
		
		//Acquire the root of the archive. It's a folder, so its contents can be
		//iterated. As will all APIs that begin with Acquire, the object will need
		//to be released.
		ArchiveObjectRef	rootObject;
		err = AcquireRootArchiveObject(archiveRef, &rootObject);
		
		//Create an iterator for the root object.
		ArchiveIteratorRef	iteratorRef;
		err = NewArchiveIterator(rootObject, &iteratorRef);
		
		//Examine the object the iterator currently points to.
		ArchiveObjectRef	currentObject;
		err = AcquireCurrentArchiveObject(iteratorRef, &currentObject);
		
		//Loop over each object the iterator references.
		while (err != kStuffItErrorNoObjectAtThatLocation)
			{
			//Get the name of the current object
			StringRef	objectName;
			err = AcquireArchiveObjectStringProperty(currentObject, kArchiveObjectPropertyIDName, &objectName);
			
			//Dispose of the name and the current object and get the next one
			err = DisposeStringRef(objectName);
			
			err = ReleaseArchiveObject(currentObject);
			
			err = AcquireNextArchiveObject(iteratorRef, &currentObject);
			}

		//And finally, dispose and release everything
		err = DisposeArchiveIterator(iteratorRef);
		
		err = ReleaseArchiveObject(rootObject);
		
		err = CloseArchive(archiveRef);
		
		err = ReleaseArchive(archiveRef);
	}
	\endcode
*/